/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import Friends 0.1

Item {
    id: details
    height: visible ? childrenRect.height + units.gu(1) : 0

    property alias hasFocus: entryArea.activeFocus
    onFocusChanged: {
        if (focus && entry && entry.inputEntry) {
            entry.inputEntry.forceActiveFocus()
        }
    }

    FriendsDispatcher {
        id: friends
        onSendComplete: {
            if (success) {
                sendSpinner.visible = false;
                entryArea.text = "";
                console.log ("Send completed successfully");
            } else {
                sendSpinner.visible = false;
                console.log ("Send failed: " + errorMessage);
            }
        }
    }

    Item {
        id: pictureWrapper
        anchors.right: parent.right
        width: parent.width - units.gu(8)
        height: childrenRect.height

        Image {
            id: pictureImage
            visible: link_picture.length > 0 && status == Image.Ready
            source: Qt.resolvedUrl(link_picture)
            asynchronous: true
            fillMode: Image.PreserveAspectCrop
            sourceSize.width: units.dp(128)
            sourceSize.height: units.dp(128)
            MouseArea {
                anchors.fill: pictureImage
                onClicked: {
                    if (link_url.length > 0)
                    {
                        Qt.openUrlExternally(link_url);
                    }
                }
            }
        }
    }

    ThreadView {
        id: threadView
        width: parent.width
        height: childrenRect.height

        anchors {
            top: pictureWrapper.bottom
            left: parent.left
            right: parent.right
        }
    }

    Item {
        id: entry
        property var inputEntry: entryArea
        property var lastCursorPosition
        visible: !from_me
        height: from_me ? 0 : childrenRect.height
        anchors {
            top: threadView.bottom
            left: parent.left
            right: parent.right
            topMargin: units.gu(2)
        }

        MouseArea {
            anchors.fill: entryArea
            onClicked: entryArea.forceActiveFocus()
            z: 1
        }

        TextArea {
            id: entryArea
            anchors {
                left: parent.left
                leftMargin: units.gu(2)
                right: sendButton.left
                rightMargin: units.gu(1)
                bottomMargin: units.gu(1.5)
            }
            //focus: false
            font.pixelSize: FontUtils.sizeToPixels("medium")
            font.family: "Ubuntu"
            // send message if return was pressed
            Keys.onReturnPressed: sendButton.clicked(null)
            Keys.onEscapePressed: text = ""
            height: from_me ? 0 : units.gu(4)
            placeholderText: i18n.tr("Reply")
            autoSize: true
            maximumLineCount: 0
            color: text.length < 140 ? "gray" : "red"
            textFormat: TextEdit.PlainText
            contentWidth: width - units.gu(5)
            onActiveFocusChanged: {
                if (activeFocus) {
                    if (service == "twitter" || service == "identica") {
                        if (entryArea.text.search("@"+sender_nick) == -1) {
                            entryArea.text = "@"+sender_nick+" ";
                            cursorPosition = text.length;
                        }
                    }
                }
            }

            onTextChanged: {
                var enteredText = text.substring(entry.lastCursorPosition, cursorPosition);
                if (enteredText.substring(0,4) == "http") {
                    var shortUrl = friends.urlShorten(enteredText);
                    if (shortUrl.length > 4) {
                        text = text.replace (enteredText, shortUrl);
                        cursorPosition = text.length;
                    }
                }
                entry.lastCursorPosition = cursorPosition;
            }

            Item {
                id: counter
                anchors {
                    right: entryArea.right
                    top: entryArea.top
                    rightMargin: units.gu(1)
                }
                height: units.gu(4)
                width: childrenRect.width
                Text {
                    text: 140 - entryArea.text.length
                    color: "gray"
                    font.family: "Ubuntu"
                    font.bold: true
                    font.pixelSize: FontUtils.sizeToPixels("large")
                    opacity: 0.3
                }
            }
        }

        Button {
            id: sendButton
            anchors {
                right: parent.right
                rightMargin: units.gu(1)
                bottom: entryArea.bottom
            }
            width: units.gu(9)
            height: from_me ? 0 : units.gu(4)
            //color: "#dd4814"
            text: i18n.tr("Send")
            onClicked: {
                console.log ("Reply to " + messageId + " from " + accountId + ": " + entryArea.text);
                sendSpinner.visible = true
                friends.sendReplyAsync(accountId, messageId, entryArea.text);
            }
        }

        ActivityIndicator {
            id: sendSpinner
            anchors.centerIn: entryArea
            visible: false
            running: visible
        }
    }
}
